/**
 * Copyright 2012, Ayuntamiento de Tarragona
 *  
 * Desarrollador por  Guadaltel, S.A.
 * 
 * El presente cdigo fue desarrollado con financiacin del Ministerio 
 * de Industria Turismo y Comercio con el nmero de proyecto TSI-050100-2009-219.
 * 
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the 
 * European Commission  subseqent versions of the EUPL (the "Licence");  you 
 * may not use this work except in compliance with the Licence.
 *
 * You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software distributed 
 * under the  Licence is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
 * CONDITIONS OF ANY KIND, either express or implied. See the Licence for the 
 * specific language governing  permissions and limitations under the Licence.
 */
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.InputStreamRequestEntity;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.log4j.Logger;

import sun.misc.BASE64Decoder;

import es.guadaltel.framework.authenticator.ICertReader;
import es.guadaltel.framework.authenticator.domain.InfoCertificado;
import es.guadaltel.framework.authenticator.exception.AuthenticatorException;
import es.guadaltel.framework.authenticator.impl.CatCertReader;
import es.guadaltel.framework.authenticator.impl.DnieCertReader;
import es.guadaltel.framework.authenticator.impl.FnmtCertReader;
import es.guadaltel.framework.authenticator.utils.AuthProperties;
import es.guadaltel.framework.authenticator.utils.UtilsWSPSIS;
import es.guadaltel.framework.core.Document;
import es.guadaltel.framework.core.User;
import es.guadaltel.framework.signer.SignTransaction;
import es.guadaltel.framework.signer.Signer;
import es.guadaltel.framework.signer.SignerFactory;


public class Prueba {
	private static Logger LOG = Logger.getLogger(Prueba.class);

	public static void main(String[] args) {
		String catcert = "MIII5zCCB8+gAwIBAgIDAkXqMA0GCSqGSIb3DQEBBQUAMIIBMTELMAkG...";
		String fnmtcert = "MIIF0DCCBTmgAwIBAgIEPMCm4zANBgkqhkiG9w0BAQUFADA2MQswCQ...";

		User userDetails = autenticar(fnmtcert);

		System.out.println(userDetails.getId()+" - "+userDetails.getName());
	}



	private static User autenticar(String certificado){
		User userDetails = new User();
		PostMethod method = null;

		try{
			userDetails.setProperty("certificado", certificado);

			InfoCertificado infoCertificado = null;

			String url = AuthProperties.getProperty("authenticator.psis.endpoint");

			HttpClient client = new HttpClient();

			StringBuffer data = new StringBuffer();
			data.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
			data.append("<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\"><soapenv:Body>");
			data.append("<urn:VerifyRequest Profile=\"urn:oasis:names:tc:dss:1.0:profiles:XSS\" xmlns:urn=\"urn:oasis:names:tc:dss:1.0:core:schema\"><urn:OptionalInputs><urn:ReturnProcessingDetails/></urn:OptionalInputs><urn:SignatureObject><urn:Other><xd:X509Data xmlns:xd=\"http://www.w3.org/2000/09/xmldsig#\"><xd:X509Certificate>");
			data.append(certificado);
			data.append("</xd:X509Certificate></xd:X509Data></urn:Other></urn:SignatureObject></urn:VerifyRequest>");
			data.append("</soapenv:Body></soapenv:Envelope>");

			LOG.debug("[PSIS Request]...\n" + data.toString());

			ByteArrayInputStream in = new ByteArrayInputStream(data.toString().getBytes());

			method = new PostMethod(url);
			InputStreamRequestEntity isRequest = new InputStreamRequestEntity(in);
			method.setRequestEntity(isRequest);

			LOG.debug("Lanzando peticion...");
			int status = client.executeMethod(method);

			LOG.debug("Peticion finalizada, codigo de estado devuelto: " + status + " (OK=200)");
			if (status == 200)
			{
				InputStream stream = null;
				stream = method.getResponseBodyAsStream();
				ByteArrayOutputStream out = UtilsWSPSIS.getData(stream);

				String result = out.toString();
				LOG.debug("[PSIS Response]...\n" + result);

				if ((result.indexOf(AuthProperties.getProperty("authenticator.psis.majorResult")) > 0) && (result.indexOf(AuthProperties.getProperty("authenticator.psis.minorResult")) > 0)) {
					infoCertificado = getInfoCertificado(certificado);

					userDetails = UtilsWSPSIS.parseUser(infoCertificado);
				}
				else {
					LOG.error("Certificado no valido (caducado o revocado)");
				}
			} else {
				LOG.error("El servidor PSIS devolvio un estado incorrecto");
			}
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			if (method != null) method.releaseConnection();
		}

		return userDetails;
	}



	private static InfoCertificado getInfoCertificado(X509Certificate certificate)
	{
		InfoCertificado result = null;

		ICertReader reader = new CatCertReader();
		if (reader.isTypeOf(certificate)) {
			LOG.debug("Recuperando informacion de certificado CatCert");
			result = reader.getInfo(certificate);
		}
		else {
			reader = new FnmtCertReader();
			if (reader.isTypeOf(certificate)) {
				LOG.debug("Recuperando informacion de certificado FNMT");
				result = reader.getInfo(certificate);
			}
			else {
				reader = new DnieCertReader();
				if (reader.isTypeOf(certificate)) {
					LOG.debug("Recuperando informacion de certificado Dnie");
					result = reader.getInfo(certificate);
				}
				else {
					LOG.debug("Tipo de certificado no soportado");
				}
			}
		}
		if (result != null) {
			LOG.debug("Extracted subject: " + result.getSubject());
		}
		return result;
	}

	private static InfoCertificado getInfoCertificado(String certificate)
	{
		X509Certificate cert = null;
		try
		{
			BASE64Decoder decoder = new BASE64Decoder();

			ByteArrayInputStream in = new ByteArrayInputStream(decoder.decodeBuffer(certificate));
			CertificateFactory factory = CertificateFactory.getInstance("X.509");
			cert = (X509Certificate)factory.generateCertificate(in);
		} catch (CertificateException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		return getInfoCertificado(cert);
	}

	public static String getAuthenticationURL(HttpServletRequest request) throws AuthenticatorException {
		LOG.debug("Entrando en el metodo getAuthenticationURL");

		String retorno = "";
		try
		{
			String idSesion = request.getSession().getId();

			if ((idSesion == null) || (idSesion.equals(""))) {
				LOG.error("Ocurrio un error al obtener el id de sesion.");
				throw new AuthenticatorException("Ocurrio un error al obtener el id de sesion.");
			}

			Signer signer = SignerFactory.createInstance();
			SignTransaction signTransaction = new SignTransaction();

			Document document = new Document();
			document.setContent(idSesion.getBytes());

			LOG.debug("Realizamos firma en dos fases para guardar signTransaction en sesion");
			signTransaction.addDocument(document);
			signTransaction.setProperty("type", "twoPhases");
			signer.initSign(signTransaction, request);

			String clientSignerConfig = (String)request.getAttribute("clientSignerConfig");

			if ((clientSignerConfig == null) || (clientSignerConfig.equals(""))) {
				LOG.error("Ocurrio un error al obtener la configuracion del cliente de firma");
				throw new AuthenticatorException("Ocurrio un error al obtener la configuracion del cliente de firma");
			}

			request.getSession().setAttribute("clientSignerConfig", clientSignerConfig);
			request.getSession().setAttribute("signTransAutenticacion", signTransaction);

			retorno = AuthProperties.getProperty("authenticator.psis.url.return");

			if ((retorno == null) || (retorno.equals("")))
				LOG.warn("Ocurrio un error al obtener la url de retorno de autenticacion");
		}
		catch (Exception e)
		{
			LOG.error("Error in method getAuthenticationURL. Error: " + e.getMessage());
			throw new AuthenticatorException(e);
		}

		LOG.debug("Saliendo del metodo getAuthenticationURL");
		return retorno;
	}
}
