/**
 * Copyright 2012, Ayuntamiento de Tarragona
 *  
 * Desarrollador por  Guadaltel, S.A.
 * El presente cdigo fue desarrollado con financiacin del Ministerio 
 * de Industria Turismo y Comercio con el nmero de proyecto TSI-050100-2009-219.
 * 
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the 
 * European Commission  subseqent versions of the EUPL (the "Licence");  you 
 * may not use this work except in compliance with the Licence.
 *
 * You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software distributed 
 * under the  Licence is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
 * CONDITIONS OF ANY KIND, either express or implied. See the Licence for the 
 * specific language governing  permissions and limitations under the Licence.
 */
package es.tarragona.psis.signer;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.TimeZone;

import oasis.names.tc.dss._1_0.core.wsdl.SOAPport;
import oasis.names.tc.dss._1_0.core.wsdl.digitalSignatureServiceClient;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlOptions;
import org.w3.x2000.x09.xmldsig.DigestMethodType;
import org.w3.x2000.x09.xmldsig.KeyInfoType;
import org.w3.x2000.x09.xmldsig.ObjectType;
import org.w3.x2000.x09.xmldsig.SignatureType;
import org.w3.x2000.x09.xmldsig.X509DataType;
import org.w3c.dom.NodeList;

import x0CoreSchema.oasisNamesTcDss1.SignRequestDocument;
import x0CoreSchema.oasisNamesTcDss1.SignResponseDocument;
import x0CoreSchema.oasisNamesTcDss1.SignatureObjectType;
import x0CoreSchema.oasisNamesTcDss1.VerifyRequestDocument;
import x0CoreSchema.oasisNamesTcDss1.VerifyResponseDocument;
import x0CoreSchema.oasisNamesTcDss1.DocumentHashDocument.DocumentHash;
import x0CoreSchema.oasisNamesTcDss1.InputDocumentsDocument.InputDocuments;
import x0CoreSchema.oasisNamesTcDss1.KeySelectorDocument.KeySelector;
import x0CoreSchema.oasisNamesTcDss1.OptionalInputsDocument.OptionalInputs;
import x0CoreSchema.oasisNamesTcDss1.ResultDocument.Result;
import x0CoreSchema.oasisNamesTcDss1.SignRequestDocument.SignRequest;
import x0CoreSchema.oasisNamesTcDss1.SignResponseDocument.SignResponse;
import x0CoreSchema.oasisNamesTcDss1.TimestampDocument.Timestamp;
import x0CoreSchema.oasisNamesTcDss1.VerifyRequestDocument.VerifyRequest;
import x0CoreSchema.oasisNamesTcDss1.VerifyResponseDocument.VerifyResponse;

import es.tarragona.psis.signer.utils.PSISConstants;
import es.tarragona.psis.signer.utils.PSISUtils;
import es.tarragona.psis.signer.utils.SignerExceptionPSIS;


public class SignerClientPSIS {
	private final Logger LOG = Logger.getLogger(SignerClientPSIS.class);

	private String certificate;
	private byte[] documentData;
	private String hashToSign;
	private SignatureObjectType signature;


	public SignResponse sign(){
		LOG.debug("Entrando en el metodo de firma en PSIS");

		SignResponse response = null;

		try{
			XmlOptions options = new XmlOptions();

			Map namespacePrefixes = new HashMap();
			namespacePrefixes.put("xss", "urn:oasis:names:tc:dss:1.0:profiles:XSS");
			namespacePrefixes.put("dss", "urn:oasis:names:tc:dss:1.0:core:schema");
			options.setSaveSuggestedPrefixes(namespacePrefixes);

			SignRequestDocument requestDocument = SignRequestDocument.Factory.newInstance(options);
			SignRequest request = requestDocument.addNewSignRequest();
			request.setProfile(PSISUtils.getSignerProperty(PSISConstants.SIGN_PROFILE));

			if(hashToSign==null)
				hashToSign = PSISUtils.getDocumentHash(documentData);

			LOG.debug("Hash del documento a firmar: "+hashToSign);
			request.setInputDocuments(PSISUtils.createInputDocumentFromHash(hashToSign));

			// Creaci de l'element amb els parmetres opcionals necessaris per generar el timestamp
			OptionalInputs optional = request.addNewOptionalInputs();
			KeySelector selector = optional.addNewKeySelector();
			KeyInfoType key = selector.addNewKeyInfo();
			X509DataType x509data = key.addNewX509Data();

			// S'afegeix el certificat de TSA de CATCert en Base64
			x509data.addX509Certificate(PSISUtils.decodeBase64String(certificate));

			optional.addSignatureType(PSISUtils.getSignerProperty(PSISConstants.SIGN_TYPE));


			// Execuci del servei
			PSISUtils.cargarContextoPeticiones();


			digitalSignatureServiceClient proxy = new digitalSignatureServiceClient();
			SOAPport port = proxy.getdssPortSoap(PSISUtils.getSignerProperty(PSISConstants.ENDPOINT));


			requestDocument = SignRequestDocument.Factory.parse(requestDocument.newInputStream());

			// Visualitzaci de la petici
			LOG.debug("[PETICION PSIS...]\n"+requestDocument.xmlText(options));
			LOG.debug("Comenzando proceso de firma en PSIS");

			SignResponseDocument responseDocument = port.sign(requestDocument);

			LOG.debug("Proceso de firma en PSIS finalizado");
			// visualitzaci de la resposta
			LOG.debug("[RESPUESTA PSIS...]\n"+responseDocument.xmlText(options));


			response = responseDocument.getSignResponse();

			if(response!=null)
				signature = response.getSignatureObject();
		}
		catch(Exception ex){
			LOG.error("ERROR - Ocurrio un error durante el proceso de firma");
			ex.printStackTrace();
		}


		LOG.debug("Saliendo del metodo de firma en PSIS");

		return response;
	}

	public boolean validateSignResponse(SignResponse response){
		LOG.debug("Validando la firma realizada en PSIS...");

		Result result = response.getResult();
		String major = result.getResultMajor();
		String minor = result.getResultMinor();

		if(major.equals(PSISUtils.getSignerProperty(PSISConstants.SIGN_MAJOR_RESULT))){
			LOG.debug("La firma se realizo correctamente");
			return true;
		}

		LOG.debug("La firma solicitada no ha podido realizarse o no es valida: "+major+"."+minor);
			return false;
	}


	public VerifyResponse verifySign(){
		LOG.debug("Entrando en el metodo de verificacion de firma en PSIS");

		VerifyResponse response = null;

		try{
			XmlOptions options = new XmlOptions();

			Map namespacePrefixes = new HashMap();
			namespacePrefixes.put("xss", "urn:oasis:names:tc:dss:1.0:profiles:XSS");
			namespacePrefixes.put("dss", "urn:oasis:names:tc:dss:1.0:core:schema");
			namespacePrefixes.put("ds", "http://www.w3.org/2000/09/xmldsig#");
			options.setSaveSuggestedPrefixes(namespacePrefixes);

			VerifyRequestDocument requestDocument = VerifyRequestDocument.Factory.newInstance(options);
			VerifyRequest request = requestDocument.addNewVerifyRequest();
			request.setProfile(PSISUtils.getSignerProperty(PSISConstants.VALIDATION_PROFILE));

			if(signature == null)
				throw new SignerExceptionPSIS("El campo de firma esta vacio");

			request.setSignatureObject(signature);

			// Creaci de l'element amb el document signat
			InputDocuments inpDocuments = request.addNewInputDocuments();

			DocumentHash document = inpDocuments.addNewDocumentHash();
			DigestMethodType digestMethod = document.addNewDigestMethod();
			digestMethod.setAlgorithm("http://www.w3.org/2000/09/xmldsig#sha1");

			if(hashToSign==null)
				hashToSign = PSISUtils.getDocumentHash(documentData);

			LOG.debug("Hash del documento a verificar: "+hashToSign);
			document.setDigestValue(PSISUtils.decodeBase64String(hashToSign));

			document.setID("docID");

			// Creaci de l'element amb els parmetres opcionals a consultar
			OptionalInputs optional = request.addNewOptionalInputs();

			// Detalls del resultat de validaci
			optional.addNewReturnProcessingDetails();

			Calendar verificationTime = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
			//Calendar verificationTime = Calendar.getInstance();
			//verificationTime.add(Calendar.HOUR, 1);

			optional.addVerificationTime(verificationTime);

			// Execuci del servei
			digitalSignatureServiceClient proxy = new digitalSignatureServiceClient();
			SOAPport port = proxy.getdssPortSoap(PSISUtils.getSignerProperty(PSISConstants.ENDPOINT));

			requestDocument = VerifyRequestDocument.Factory.parse(requestDocument.newInputStream());


			PSISUtils.cargarContextoPeticiones();


			// Visualitzaci de la petici
			LOG.debug("[PETICION PSIS...]\n"+requestDocument.xmlText());
			LOG.debug("Comenzando proceso de verificacion de firma en PSIS");

			VerifyResponseDocument responseDocument = port.verify(requestDocument);

			LOG.debug("Proceso de verificacion de firma en PSIS finalizado");
			// visualitzaci de la resposta
			LOG.debug("[RESPUESTA PSIS...]\n"+responseDocument.getVerifyResponse().xmlText());


			response = responseDocument.getVerifyResponse();
		}
		catch(Exception ex){
			LOG.error("ERROR - Ocurrio un error durante la verificacion de la firma");
			ex.printStackTrace();
		}


		LOG.debug("Saliendo del metodo de verificacion de firma en PSIS");

		return response;
	}

	public boolean validateVerifyResponse(VerifyResponse response){
		Result result = response.getResult();
		String major = result.getResultMajor();
		String minor = result.getResultMinor();

		if(major.equals(PSISUtils.getSignerProperty(PSISConstants.VALIDATION_MAJOR_RESULT)) &&
				minor.equals(PSISUtils.getSignerProperty(PSISConstants.VALIDATION_MINOR_RESULT))){
			LOG.debug("Firma verificada correctamente");
			return true;
		}

		LOG.debug("La firma no es correcta: "+major+". "+minor);
			return false;
	}

	public void setCertificate(String certificado){
		certificate = certificado;
	}

	public void setDocumentData(byte[] data){
		documentData = data;
	}

	public void setHashToSign(String hash){
		hashToSign = hash;
	}

	public void importSignFromReport(String signReport){
		try{
			XmlOptions options = new XmlOptions();

			Map namespacePrefixes = new HashMap();
			namespacePrefixes.put("xss", "urn:oasis:names:tc:dss:1.0:profiles:XSS");
			namespacePrefixes.put("dss", "urn:oasis:names:tc:dss:1.0:core:schema");
			namespacePrefixes.put("ds", "http://www.w3.org/2000/09/xmldsig#");
			options.setSaveSuggestedPrefixes(namespacePrefixes);

			String signReportFinal = "<dss:Timestamp xmlns:dss=\"urn:oasis:names:tc:dss:1.0:core:schema\">"+signReport+"</dss:Timestamp>";
			signature = SignatureObjectType.Factory.parse(signReportFinal, options);
		}
		catch(XmlException ex){
			LOG.error("Se produjo un error al parsear la firma");
			ex.printStackTrace();
		}
	}



	public Date getSignTimeStamp(Timestamp timestamp){
		Date fecha = null;

		try{
			SignatureType sigType = timestamp.getSignature();

			ObjectType []objs = sigType.getObjectArray();

			for(ObjectType obj : objs){
				NodeList nodos = obj.getDomNode().getChildNodes();

				for(int i=0; i<nodos.getLength(); i++){
					if(nodos.item(i).getNodeName().toUpperCase().contains("TSTINFO")){
						NodeList nodosTime = nodos.item(i).getChildNodes();

						for(int j=0; j<nodosTime.getLength(); j++){
							if(nodosTime.item(j).getNodeName().toUpperCase().contains("CREATIONTIME")){
								String time = nodosTime.item(j).getFirstChild().getNodeValue();
								String finaltime = time.replace("T", " ").replace("Z", "");

								SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
								fecha = df.parse(finaltime);
								break;
							}
						}
					}
				}
			}
		}catch(Exception e){
			LOG.error("Ocurrio un error al recuperar el timestamp");
			e.printStackTrace();
		}

		return fecha;
	}

	public Timestamp getSignTimeStampResult() throws SignerExceptionPSIS{
		if(signature!=null)
			return signature.getTimestamp();

		throw new SignerExceptionPSIS("El campo de firma esta vacio");
	}

	public void clearData(){
		certificate = null;
		documentData = null;
		signature = null;
	}
}