/**
 * Copyright 2012, Ayuntamiento de Tarragona
 *  
 * Desarrollador por  Guadaltel, S.A.
 * El presente cdigo fue desarrollado con financiacin del Ministerio 
 * de Industria Turismo y Comercio con el nmero de proyecto TSI-050100-2009-219.
 * 
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the 
 * European Commission  subseqent versions of the EUPL (the "Licence");  you 
 * may not use this work except in compliance with the Licence.
 *
 * You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software distributed 
 * under the  Licence is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
 * CONDITIONS OF ANY KIND, either express or implied. See the Licence for the 
 * specific language governing  permissions and limitations under the Licence.
 */
package es.tarragona.psis.signer.utils;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.security.Security;
import java.util.ResourceBundle;

import javax.servlet.http.HttpServletRequest;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.codec.digest.DigestUtils;
import org.apache.log4j.Logger;
import org.w3.x2000.x09.xmldsig.DigestMethodType;

import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;
import x0CoreSchema.oasisNamesTcDss1.DocumentHashDocument.DocumentHash;
import x0CoreSchema.oasisNamesTcDss1.InputDocumentsDocument.InputDocuments;


public class PSISUtils {
	private static Logger LOG = Logger.getLogger(PSISUtils.class);

	public static byte[] decodeBase64String(final String content) throws Exception {
		BASE64Decoder b64 = new BASE64Decoder();
		return b64.decodeBuffer(content);
	}

	public static String readFile(final String filename) throws Exception {
		StringBuffer sb = new StringBuffer();

		//ClassLoader cl = Thread.currentThread().getContextClassLoader();
		//DataInputStream dis = new DataInputStream(cl.getResourceAsStream(filename));
		DataInputStream dis = new DataInputStream(new FileInputStream(new File(filename)));

		byte[] buffer = new byte[4096];
		for (int len; (len = dis.read(buffer)) > 0;) {
			sb.append(new String(buffer, 0, len));
		}
		dis.close();

		return sb.toString();
	}

	public static byte[] readBase64File(final String filename) throws Exception {
		return (decodeBase64String(readFile(filename)));
	}

	public static String readXmlFile(final String filename) throws Exception {
		return readFile(filename);
	}

	public static String getSignerProperty(String key){
		ResourceBundle propiedades = ResourceBundle.getBundle("signer");
		String result = null;

		try{
			result = propiedades.getString(key);
		}catch(Exception ex){
			LOG.debug("No se pudo encontrar la propiedad: "+key);
			ex.printStackTrace();
		}

		return result;
	}

	public static void cargarContextoPeticiones(){
		// Configuraci socket TSL

		// Cert must be added to cacerts using keytool or specified here
		System.setProperty("javax.net.ssl.trustStore", PSISUtils.getSignerProperty(PSISConstants.TRUSTEDSTORE));
		System.setProperty("javax.net.ssl.trustStoreType", PSISUtils.getSignerProperty(PSISConstants.TRUSTEDSTORE_TYPE));
		System.setProperty("javax.net.ssl.trustStorePassord", PSISUtils.getSignerProperty(PSISConstants.TRUSTEDSTORE_PASSWORD));

		// Security Provider
		System.setProperty("java.protocol.handler.pkgs","com.sun.net.ssl.internal.www.protocol");
		Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());
	}


	public static InputDocuments createInputDocumentFromHash(String hashData){
		InputDocuments inpDocuments = null;

		try{
			// Creaci de l'element amb el document per generar el timestamp
			inpDocuments = InputDocuments.Factory.newInstance();

			DocumentHash document = inpDocuments.addNewDocumentHash();
			DigestMethodType digestMethod = document.addNewDigestMethod();
			digestMethod.setAlgorithm("http://www.w3.org/2000/09/xmldsig#sha1");
			document.setDigestValue(decodeBase64String(hashData));
			document.setID("docID");

		}catch(Exception ex){
			ex.printStackTrace();
		}

		return inpDocuments;
	}

	public static String getDocumentHash(byte[] documentContent){
		byte[] hash = DigestUtils.sha(documentContent);

		BASE64Encoder b64 = new BASE64Encoder();
		String newhash = b64.encode(hash);

		return newhash;
	}

	public static String getCertificateFromRequest(HttpServletRequest request) throws SignerExceptionPSIS{
		String csResult = (String) request.getParameter("clientSignerResult");
		if (csResult == null || csResult.equals("")) {
			LOG.error ("Ocurrio un error al obtener inputData.clientSignerResult");
            throw new SignerExceptionPSIS("Ocurrio un error al obtener inputData.clientSignerResult");
        }

		JSONArray jsonArray = JSONArray.fromObject(csResult);
        if (jsonArray == null) {
            LOG.error("Ocurrio un error al obtener la informacion de firma");
            throw new SignerExceptionPSIS("Ocurrio un error al obtener la informacion de firma");
        }

        String certificate = (String) ((JSONObject)jsonArray.get(jsonArray.size()-1)).get("certificate");
        if (certificate == null || certificate.equals("")) {
        	LOG.error("Se ha producido un error en el metodo sign. El parametro certificate es necesario");
        	throw new SignerExceptionPSIS("Error en el metodo sign. El parametro es necesario");
        }

        return certificate;
	}
}