/**
 * Copyright 2012, Ayuntamiento de Tarragona
 *  
 * Desarrollador por  Guadaltel, S.A.
 * El presente cdigo fue desarrollado con financiacin del Ministerio 
 * de Industria Turismo y Comercio con el nmero de proyecto TSI-050100-2009-219.
 * 
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the 
 * European Commission  subseqent versions of the EUPL (the "Licence");  you 
 * may not use this work except in compliance with the Licence.
 *
 * You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software distributed 
 * under the  Licence is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
 * CONDITIONS OF ANY KIND, either express or implied. See the Licence for the 
 * specific language governing  permissions and limitations under the Licence.
 */
package es.tarragona.psis.signer;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;
import java.util.TimeZone;

import oasis.names.tc.dss._1_0.core.wsdl.SOAPport;
import oasis.names.tc.dss._1_0.core.wsdl.digitalSignatureServiceClient;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlOptions;
import org.w3.x2000.x09.xmldsig.DigestMethodType;

import x0CoreSchema.oasisNamesTcDss1.VerifyRequestDocument;
import x0CoreSchema.oasisNamesTcDss1.VerifyResponseDocument;
import x0CoreSchema.oasisNamesTcDss1.DocumentHashDocument.DocumentHash;
import x0CoreSchema.oasisNamesTcDss1.InputDocumentsDocument.InputDocuments;
import x0CoreSchema.oasisNamesTcDss1.OptionalInputsDocument.OptionalInputs;
import x0CoreSchema.oasisNamesTcDss1.SignResponseDocument.SignResponse;
import x0CoreSchema.oasisNamesTcDss1.TimestampDocument.Timestamp;
import x0CoreSchema.oasisNamesTcDss1.VerifyRequestDocument.VerifyRequest;
import x0CoreSchema.oasisNamesTcDss1.VerifyResponseDocument.VerifyResponse;

import es.tarragona.psis.signer.utils.PSISConstants;
import es.tarragona.psis.signer.utils.PSISUtils;
import es.tarragona.psis.signer.utils.SignerExceptionPSIS;


public class Prueba {
	private final static Logger LOG = Logger.getLogger(Prueba.class);

	public static void main(String[] args) throws Exception {

		//very();

		String document = "/home/juliomaqueda/Escritorio/amazon/soapfinal";

		// Certificat amb que es generar la signatura o timestamp
		String certificate = PSISUtils.readFile("/home/user/repositorio/keystore/psis/timestamp-certificate.dat");
		String fileContent = PSISUtils.readFile(document);


		SignerClientPSIS client = new SignerClientPSIS();
		client.setCertificate(certificate);
		client.setDocumentData(fileContent.getBytes());


		//Firma
		SignResponse response = client.sign();

		if(response!=null && client.validateSignResponse(response)){
			Timestamp timestampResult = client.getSignTimeStampResult();
			Date fecha = client.getSignTimeStamp(timestampResult);

			System.out.println("Fecha: "+fecha.toString());


			//Verificacion
			String firmado = timestampResult.xmlText();

			client.clearData();
			client.setDocumentData(fileContent.getBytes());
			client.importSignFromReport(firmado);


			VerifyResponse vresponse = client.verifySign();

			client.validateVerifyResponse(vresponse);
		}
	}

	public static void very(){
		LOG.debug("Entrando en el metodo de verificacion de firma en PSIS");

		VerifyResponse response = null;

		try{
			InputStream is = new FileInputStream(new File("/home/user/Escritorio/prueba.dat"));
			VerifyRequestDocument requestDocument = VerifyRequestDocument.Factory.parse(is);

			// Execuci del servei
			digitalSignatureServiceClient proxy = new digitalSignatureServiceClient();
			SOAPport port = proxy.getdssPortSoap("https://psis.catcert.net/psis/catcert/dss");

			PSISUtils.cargarContextoPeticiones();


			// Visualitzaci de la petici
			//LOG.debug("[PETICION PSIS...]\n"+requestDocument.xmlText());
			LOG.debug("Comenzando proceso de verificacion de firma en PSIS");

			VerifyResponseDocument responseDocument = port.verify(requestDocument);

			LOG.debug("Proceso de verificacion de firma en PSIS finalizado");
			// visualitzaci de la resposta
			LOG.debug("[RESPUESTA PSIS...]\n"+responseDocument.getVerifyResponse().xmlText());


			response = responseDocument.getVerifyResponse();
		}
		catch(Exception ex){
			LOG.error("ERROR - Ocurrio un error durante la verificacion de la firma");
			ex.printStackTrace();
		}


		LOG.debug("Saliendo del metodo de verificacion de firma en PSIS");
	}
}