/**
 * Copyright 2012, Ayuntamiento de Tarragona
 *  
 * Desarrollador por  Guadaltel, S.A.
 * El presente cdigo fue desarrollado con financiacin del Ministerio 
 * de Industria Turismo y Comercio con el nmero de proyecto TSI-050100-2009-219.
 * 
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the 
 * European Commission  subseqent versions of the EUPL (the "Licence");  you 
 * may not use this work except in compliance with the Licence.
 *
 * You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software distributed 
 * under the  Licence is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
 * CONDITIONS OF ANY KIND, either express or implied. See the Licence for the 
 * specific language governing  permissions and limitations under the Licence.
 */
package es.guadaltel.framework.register.impl;

import ieci.tecdoc.sgm.core.services.dto.Entidad;
import ieci.tecdoc.sgm.registropresencial.ws.server.BookId;
import ieci.tecdoc.sgm.registropresencial.ws.server.Document;
import ieci.tecdoc.sgm.registropresencial.ws.server.Documents;
import ieci.tecdoc.sgm.registropresencial.ws.server.FieldInfo;
import ieci.tecdoc.sgm.registropresencial.ws.server.Fields;
import ieci.tecdoc.sgm.registropresencial.ws.server.Folder;
import ieci.tecdoc.sgm.registropresencial.ws.server.PersonInfo;
import ieci.tecdoc.sgm.registropresencial.ws.server.RegisterInfo;
import ieci.tecdoc.sgm.registropresencial.ws.server.ServicioRegistroWebServiceServiceLocator;
import ieci.tecdoc.sgm.registropresencial.ws.server.UserInfo;

import java.io.Serializable;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import es.guadaltel.framework.register.Register;
import es.guadaltel.framework.register.RegisterTransaction;
import es.guadaltel.framework.register.constants.RegisterSigemConstants;
import es.guadaltel.framework.register.exception.RegisterException;
import es.guadaltel.framework.register.util.Utils;


public class SigemImpl implements Serializable, Register {

	private static final long serialVersionUID = 28542094281095451L;
	private static final Log log = LogFactory.getLog(SigemImpl.class);

	private Configuration config;
	private ArrayList<FieldInfo> fieldinfo; /*Arraylist de campos del Registro*/


	public SigemImpl (Configuration configNotificator) {
		log.info ("Inicializando la implementación SIGEM del componente register... ");
		config = configNotificator;
	}

	public void register(RegisterTransaction register) throws RegisterException {
	    if (register == null) {
            log.error ("El parametro registerTransaction del metodo register no es valido.");
            throw new RegisterException ("El parametro registerTransaction del metodo register no es valido.");
        }

	    String type = register.getType(); //entry or out


	    //ENTIDAD
        Entidad entidad = new Entidad();
        entidad.setIdentificador(getNotificatorProperty(RegisterSigemConstants.REGISTER_ENTIDAD));


        //INFORMACION DEL USUARIO (SE DARA DE ALTA EN LA OFICINA POR DEFECTO DEL USUARIO)
        UserInfo user = new UserInfo();
        user.setUserName(getNotificatorProperty(RegisterSigemConstants.REGISTER_PRES_USUARIO));
        user.setPassword(getNotificatorProperty(RegisterSigemConstants.REGISTER_PRES_PASS));
        user.setLocale(getNotificatorProperty(RegisterSigemConstants.REGISTER_PRES_USER_LOCALE));


        //LIBRO DEL REGISTRO
        BookId book = new BookId();

        if(type.equals(RegisterSigemConstants.TYPE_REGISTER_ENTRY)){ //Registro de entrada
            book.setBookId(getNotificatorProperty(RegisterSigemConstants.REGISTER_PRES_BOOK_IN));

            if (register.getExtract() != null){
                if (register.getExtract().length() >= 237)
                    register.setExtract(Utils.shortString(register.getExtract(), 236));
            }else{
                log.info("No se ha incluido extracto, se usa el texto configurado por defecto.");
                register.setExtract(getNotificatorProperty(RegisterSigemConstants.REGISTER_PRES_E_SUBJECT_DEFAULT));
            }

            //Insertamos el destinatario
            addFieldInfo(   getNotificatorProperty(RegisterSigemConstants.REGISTER_PRES_CAMPO_E_DESTINO),
                            getNotificatorProperty(RegisterSigemConstants.REGISTER_ORGANIZATION));

            //Insertamos el tipo de asunto
            addFieldInfo(   getNotificatorProperty(RegisterSigemConstants.REGISTER_PRES_CAMPO_E_TASUNTO),
                            getNotificatorProperty(RegisterSigemConstants.REGISTER_SUBJECT));

            //Insertamos el resumen
            addFieldInfo(   getNotificatorProperty(RegisterSigemConstants.REGISTER_PRES_CAMPO_E_RESUMEN),
                            register.getExtract());
        }
        else{ //Registro de salida
            book.setBookId(getNotificatorProperty(RegisterSigemConstants.REGISTER_PRES_BOOK_OUT));

            //Insertamos el origen
            addFieldInfo(   getNotificatorProperty(RegisterSigemConstants.REGISTER_PRES_CAMPO_S_ORIGEN),
                            getNotificatorProperty(RegisterSigemConstants.REGISTER_ORGANIZATION));

            //Insertamos el tipo de transporte
            addFieldInfo(   getNotificatorProperty(RegisterSigemConstants.REGISTER_PRES_CAMPO_S_TTRANSPORTE),
                            getNotificatorProperty(RegisterSigemConstants.REGISTER_PRES_S_TIP_TRAN));

            //Insertamos el tipo de asunto
            addFieldInfo(   getNotificatorProperty(RegisterSigemConstants.REGISTER_PRES_CAMPO_S_TASUNTO),
                            getNotificatorProperty(RegisterSigemConstants.REGISTER_SUBJECT));
        }

        FieldInfo[] atts = Utils.toArray(fieldinfo);


        //PERSONAS
        PersonInfo[] arrayRemitentes = new PersonInfo[1];

        PersonInfo remitente = new PersonInfo();

        if(type.equals(RegisterTransaction.TYPE_REGISTER_ENTRY))
            remitente.setPersonName(register.getSender().getName());
            //remitente.setPersonName("47762271B - MEDRANO MARTORELL, TXOMIN");
            //remitente.setPersonId("999640");
            //remitente.setDirection("RB VELLA 2, 1r");
            //remitente.setDomId("9001200");

        arrayRemitentes[0] = remitente;


        //DOCUMENTOS
        Documents documents = new Documents();
        Document[] docArray = new Document[0];
        documents.setDocuments(docArray);


        //CAMPOS
        Fields fields = new Fields();
        fields.setFields(atts);

        Folder folder = new Folder();
        folder.setBookId(book);
        folder.setDocumentos(documents);
        folder.setFields(fields);


        //SERVICIO
        log.debug("Creando instancia del cliente de servicio web");
        ServicioRegistroWebServiceServiceLocator oLocator = new ServicioRegistroWebServiceServiceLocator();
        oLocator.setServicioRegistroWebServiceEndpointAddress(getNotificatorProperty(RegisterSigemConstants.REGISTER_PRES_WSDLLOCATION));

        RegisterInfo registerInfo = new RegisterInfo();

        try {
            log.debug("Lanzando peticion a Sigem...");
            registerInfo = oLocator.getServicioRegistroWebService().createFolder(user, arrayRemitentes, folder, entidad);

            //Validacion de la respuesta
            if (registerInfo == null) {
                log.error ("La informacion de registro resultante no es valida.");
                throw new RegisterException ("La informacion de registro resultante no es valida.");
            }

            if (registerInfo.getErrorCode() != null && !registerInfo.getErrorCode().equals("")) {
                log.error ("Se ha producido un error en el proceso de registro. Codigo de error: " + registerInfo.getErrorCode());
                throw new RegisterException ("Se ha producido un error en el proceso de registro. Codigo de error: " + registerInfo.getErrorCode());
            }

            if (registerInfo.getNumber() == null || registerInfo.getNumber().equals("") ||
                registerInfo.getDate() == null || registerInfo.getDate().equals("")) {
                log.error ("El registro de salida no se ha realizado correctamente. ");
                throw new RegisterException ("El registro de salida no se ha realizado correctamente.");
            }

            log.debug("Registro en Sigem realizado correctamente. Number: "+registerInfo.getNumber()+". Fecha: "+registerInfo.getDate());

            //Se parsea la fecha
            String dateFormat = getNotificatorProperty(RegisterSigemConstants.REGISTER_PRES_DATEFORMAT);
            SimpleDateFormat parseDate = new SimpleDateFormat(dateFormat);
            Date dateFechaReg = parseDate.parse(registerInfo.getDate());
            Timestamp tsFechaReg = new Timestamp(dateFechaReg.getTime());

            //Asignamos los valores del registro en Sigem
            register.setRecordDate(tsFechaReg);
            register.setRecordNumber(registerInfo.getNumber());

        } catch (Exception e) {
            e.printStackTrace();
            throw new RegisterException (e);
        }
	}

    private String getNotificatorProperty(String clave) throws RegisterException{
        try
        {
            if(config.getString(clave)!=null)
                return config.getString(clave);
            else{
                ResourceBundle propiedades = ResourceBundle.getBundle("register");
                return propiedades.getString(RegisterSigemConstants.PREFIX_COMPONENTE+clave);
            }
        }
        catch(MissingResourceException mre)
        {
            log.error("No se encontro la propiedad "+RegisterSigemConstants.PREFIX_COMPONENTE+clave+" en el archivo de configuracion");
            throw new RegisterException(mre);
        }
    }

    private void addFieldInfo(String id, String value){
        if(fieldinfo==null) fieldinfo=new ArrayList<FieldInfo>();
        FieldInfo fieldInfo = new FieldInfo();
        fieldInfo.setFieldId(id);
        fieldInfo.setValue(value);
        fieldinfo.add(fieldInfo);
    }
}