/**
 * Copyright 2012, Ayuntamiento de Tarragona
 *  
 * Desarrollador por  Guadaltel, S.A.
 * El presente código fue desarrollado con financiación del Ministerio ç
 * de Industria Turismo y Comercio con el número de proyecto TSI-050100-2009-219.
 * 
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the 
 * European Commission – subseqüent versions of the EUPL (the "Licence");  you 
 * may not use this work except in compliance with the Licence.
 *
 * You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software distributed 
 * under the  Licence is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
 * CONDITIONS OF ANY KIND, either express or implied. See the Licence for the 
 * specific language governing  permissions and limitations under the Licence.
 */

package es.guadaltel.framework.register.test;

import ieci.tecdoc.sgm.core.services.dto.Entidad;
import ieci.tecdoc.sgm.registropresencial.ws.server.BookId;
import ieci.tecdoc.sgm.registropresencial.ws.server.Document;
import ieci.tecdoc.sgm.registropresencial.ws.server.Documents;
import ieci.tecdoc.sgm.registropresencial.ws.server.FieldInfo;
import ieci.tecdoc.sgm.registropresencial.ws.server.Fields;
import ieci.tecdoc.sgm.registropresencial.ws.server.Folder;
import ieci.tecdoc.sgm.registropresencial.ws.server.PersonInfo;
import ieci.tecdoc.sgm.registropresencial.ws.server.RegisterInfo;
import ieci.tecdoc.sgm.registropresencial.ws.server.ServicioRegistroWebServiceServiceLocator;
import ieci.tecdoc.sgm.registropresencial.ws.server.UserInfo;

import java.rmi.RemoteException;
import java.util.ArrayList;

public class TestSigemImpl {

    private static final String URL_REGISTRO_SIGEM ="http://localhost:5050/SIGEM_RegistroPresencialWS/services/ServicioRegistroWebService";
    private static final String USUARIOSIGEM = "sigemomac";
    private static final String PWDSIGEM = "sigem";

    private static final String LOCALESIGEM     =   "es";
    private static final String BOOKINSIGEM     =   "010";
    private static final String BOOKOUTSIGEM    =   "011";
    private static final String ENTIDADSIGEM    =   "000";

    private static final String TIPO_REGISTRO       =   "S";  /* E o S (entrada o salida) */

    private static ArrayList<FieldInfo> fieldinfo; /*arraylist de campos del Registro*/

    /**
     * @param args
     */
    public static void main(String[] args) {
        try {
            ServicioRegistroWebServiceServiceLocator oLocator = new ServicioRegistroWebServiceServiceLocator();
            oLocator.setServicioRegistroWebServiceEndpointAddress(URL_REGISTRO_SIGEM);

            //ENTIDAD
            Entidad entidad = new Entidad();
            entidad.setIdentificador(ENTIDADSIGEM);


            //INFORMACIÃ“N DEL USUARIO (SE DARÃ� DE ALTA EN LA OFICINA POR DEFECTO DEL USUARIO)
            UserInfo user = new UserInfo();
            user.setUserName(USUARIOSIGEM);
            user.setPassword(PWDSIGEM);
            user.setLocale(LOCALESIGEM);


            // LIBRO DEL REGISTRO
            BookId book = new BookId();
            book.setBookId(BOOKOUTSIGEM); //Registro de entrada
            Integer bookId = new Integer(BOOKINSIGEM);

            //CAMPOS

            if(TIPO_REGISTRO.equals("S")){
                addFieldInfo("7","02.05");//origen (o cÃ³digo de archivo) Ha de ser vÃ¡lido, es decir, existir en  sigem como unidad administrativa ( p:ex: 03.02 es COMPRES,  02.05 es CONTRACTACIO)
                addFieldInfo("10","PRESENCIAL");//tipo transporte (ha de existir en la tabla de tipos de transporte de SIGEM : BUROFAX, CORREU ORDINARI, PRESENCIAL, TELEMATIC
                //addFieldInfo("13","bla bla bla");//resumen
                //addFieldInfo("16","NO");//tarjeta rosa . Indica si se generarÃ¡ tarjeta rosa de correos desde nuestra aplicaciÃ³n de tarjetas rosas(SI o NO)
                addFieldInfo("12","110");// tipo de asunto . Ha de ser un tipo vÃ¡lido de la tabla de tipos de asunto de SIGEM

            }else{
                addFieldInfo("8","02.05"); //destino (o cÃ³digo de archivo) Ha de ser vÃ¡lido, es decir, existir en  sigem como unidad administrativa ( p:ex: 03.02 es COMPRES,  02.05 es CONTRACTACIO)
                addFieldInfo("16","110");//tipo asunto
                addFieldInfo("17","Assumpte del registre de prova");//resumen
                //addFieldInfo("26","TMEDRANO");//usuarialta
            }

            FieldInfo[] atts=toArray(fieldinfo);

            //PERSONAS
            PersonInfo[] arrayRemitentes =new PersonInfo[1];

            PersonInfo remitente = new PersonInfo();
            remitente.setPersonName("47762271B - MEDRANO MARTORELL, TXOMIN");
            remitente.setPersonId("999640");
            remitente.setDirection("RB VELLA 2, 1r");
            remitente.setDomId("9001200");

            arrayRemitentes[0] = remitente;


            //DOCUMENTOS
//          Documents documents = new Documents();
//          Document[] docArray = new Document[1];
//          Document doc=new Document();
//          doc.setDocumentContentB64(""); //lo paso vacio
//          doc.setDocumentName("nom_document");
//          doc.setExtension("pdf");
//          doc.setFileName("nom_document");
//          docArray[0]=doc;
//          documents.setDocuments(docArray);
            Documents documents = new Documents();
            Document[] docArray = new Document[0];
            documents.setDocuments(docArray);


            Fields fields = new Fields();
            fields.setFields(atts);

            Folder folder = new Folder();
            folder.setBookId(book);
            folder.setDocumentos(documents);
            folder.setFields(fields);


            RegisterInfo response = new RegisterInfo();
            try {
                response = oLocator.getServicioRegistroWebService().createFolder(user, arrayRemitentes, folder, entidad);

                //en caso contrario en la respuesta vienen los datos del registro generado
                response.getNumber(); //serÃ­a el nÃºmero del registro generado
                response.getWorkDate(); //serÃ­a la fecha de trabajo del registro generado
                response.getDate(); //serÃ­a la fecha interna de altra del registro generado
                response.getOffice(); //serÃ­a la oficina en la que se ha dado de alta
                response.getUserName(); //serÃ­a el usuario que ha dado de alta  (el que hemos pasado como UserInfo)
                response.getBookId();   //serÃ­a el libro en que se ha dado de alta
            } catch (RemoteException e) {
                System.out.println("ERROR ->> " + e.getMessage());
//              logger.error("WSSIGEM:"+e.getMessage());
            }

            System.out.println("FIN");
        } catch (Exception e) {
            System.out.println("ERROR");
        }
    }

    /** funciÃ³n para aÃ±adir campos FieldInfo al array de campos */
    public static void addFieldInfo(String id, String value){
        if(fieldinfo==null) fieldinfo=new ArrayList<FieldInfo>();
        FieldInfo fieldInfo = new FieldInfo();
        fieldInfo.setFieldId(id);
        fieldInfo.setValue(value);
        fieldinfo.add(fieldInfo);
    }

    /** funciÃ³n para pasar de ArrayList a array normal */
    public static FieldInfo[] toArray(ArrayList<FieldInfo> in){
        if(in==null) return new FieldInfo[0];
        if(in.size()==0) return new FieldInfo[0];
        FieldInfo[] ret=new FieldInfo[in.size()];
        for(int i=0;i<in.size();i++){
            ret[i]=(FieldInfo)in.get(i);
        }
        return ret;
    }
}
