/**
 * Copyright 2012, Ayuntamiento de Tarragona
 *  
 * Desarrollador por  Guadaltel, S.A.
 * El presente cdigo fue desarrollado con financiacin del Ministerio 
 * de Industria Turismo y Comercio con el nmero de proyecto TSI-050100-2009-219.
 * 
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the 
 * European Commission  subseqent versions of the EUPL (the "Licence");  you 
 * may not use this work except in compliance with the Licence.
 *
 * You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software distributed 
 * under the  Licence is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
 * CONDITIONS OF ANY KIND, either express or implied. See the Licence for the 
 * specific language governing  permissions and limitations under the Licence.
 */
package es.guadaltel.framework.notificator.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import net.aocat.nt.v2_1.ntresposta.DadesNotificacio;
import net.aocat.nt.v2_1.ntresposta.NTResposta;
import net.gencat.scsp.esquemes.peticion.Peticion;
import net.gencat.scsp.esquemes.respuesta.Respuesta;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.apache.commons.configuration.Configuration;
import org.w3c.dom.Document;

import es.guadaltel.framework.core.User;
import es.guadaltel.framework.notificator.NotificationState;
import es.guadaltel.framework.notificator.Notificator;
import es.guadaltel.framework.notificator.NotificatorTransaction;
import es.guadaltel.framework.notificator.State;
import es.guadaltel.framework.notificator.exception.NotificatorException;
import es.guadaltel.framework.notificator.util.NTUtils;
import es.guadaltel.framework.notificator.util.SiriClient;
import es.guadaltel.framework.notificator.util.SiriUtils;

public class EnotumImpl implements Serializable,Notificator {

    private static final long serialVersionUID = 3578480828598389758L;

    private static final Log LOG = LogFactory.getLog(EnotumImpl.class);

    //En esta implementacion por defecto se usan estos estados
    private List<String> stateAvailable = new ArrayList<String>();

    private List<String> stateRead = new ArrayList<String>();

    private List<String> stateRejected = new ArrayList<String>();

    private List<String> stateNotSubscribed = new ArrayList<String>();

    /**
     * Initialize notification component
     * @param config notification configuration
     */
    public EnotumImpl(Configuration config){
            LOG.debug("[ENOTUM] Inicializamos el componente de notificaciones a Enotum");

            //Relación de estados para la agenda trew@

            //Available
            stateAvailable.add("0");//creada
            stateAvailable.add("1");//enviada
            stateAvailable.add("2");//puesta a disposicion

            //Read
            stateRead.add("3");//leida
            stateRead.add("6");//finalizada

            //Rejected
            stateRejected.add("4");//rechazada
            stateRejected.add("5");//expirada despues de 10 dias

            //Not subscribed
            stateNotSubscribed.add("7");//usuario no suscrito
    }

    public void send(NotificatorTransaction notificatorTransaction) throws NotificatorException {
        LOG.debug("[ENOTUM] Entrando al metodo de envio de notificaciones a Enotum");

        //Peticion siri
        LOG.debug("[ENOTUM] Creando peticion Siri ...");
        Peticion peticion = SiriUtils.crearPeticionSiriPorDefecto();

        //Datos especificos de Enotum
        LOG.debug("[ENOTUM] Creando solicitud para Enotum ...");
        Document solicitudEnotum = NTUtils.crearDocumentoProcesarTramesa(notificatorTransaction);

        if(solicitudEnotum!=null){
            LOG.debug("[ENOTUM] Solicitud para Enotum creada ...");
            peticion.getSolicitudes().getSolicitudTransmision().get(0).setDatosEspecificos(solicitudEnotum.getDocumentElement());

            //Lanzamos peticion
            LOG.debug("[ENOTUM] Lanzando peticion ...");
            SiriClient enotumService = new SiriClient();
            Respuesta respuestaSiri = enotumService.enviarPeticionSiri(peticion);

            //Se comprueban los resultados
            LOG.debug("[ENOTUM] Comprobando y validando la respuesta ...");
            if(respuestaSiri!=null && SiriUtils.validarRespuestaSiri(respuestaSiri)){
                LOG.debug("[ENOTUM] Respuesta desde Siri recibida correctamente ...");
                LOG.debug("[ENOTUM] Recuperando respuesta de Enotum ...");
                NTResposta respuestaEnotum = NTUtils.recuperarRespuestaEnotum(respuestaSiri);

                if(respuestaEnotum!=null && NTUtils.validarRespuestaEnotum(respuestaEnotum)){
                    LOG.debug("[ENOTUM] Alta realizada correctamente, se almacena el id de la notificacion");

                    notificatorTransaction.setId(NTUtils.recuperarIdNotificacionEnotum(respuestaEnotum));
                }
                else
                    LOG.error("[ENOTUM] Error al recuperar la respuesta de Enotum ...");
            }
            else
                LOG.debug("[ENOTUM] Error al recuperar o validar la respuesta de Siri ...");
        }
        else
            LOG.error("[ENOTUM] Error al crear la solicitud para Enotum ...");

        LOG.debug("[ENOTUM] Saliendo del metodo de envio de notificaciones a Enotum");
    }

    public NotificationState getNotificationState(String id) throws NotificatorException {
        LOG.debug("[ENOTUM] Entrando al metodo de recuperacion de estado de Enotum");

        NotificationState notificationState = new NotificationState();
        notificationState.setState(State.UNKNOWN);

        //Peticion siri
        LOG.debug("[ENOTUM] Creando peticion Siri ...");
        Peticion peticion = SiriUtils.crearPeticionSiriPorDefecto();

        //Datos especificos de Enotum
        LOG.debug("[ENOTUM] Creando solicitud para Enotum ...");
        Document solicitudEnotum = NTUtils.crearDocumentoConsultaNotificacion(Long.parseLong(id));

        if(solicitudEnotum!=null){
            LOG.debug("[ENOTUM] Solicitud para Enotum creada ...");
            peticion.getSolicitudes().getSolicitudTransmision().get(0).setDatosEspecificos(solicitudEnotum.getDocumentElement());
        }
        else
            LOG.error("[ENOTUM] Error al crear la solicitud para Enotum ...");

        //Lanzamos peticion
        LOG.debug("[ENOTUM] Lanzando peticion ...");
        SiriClient enotumService = new SiriClient();
        Respuesta respuestaSiri = enotumService.enviarPeticionSiri(peticion);

        //Se comprueban los resultados
        LOG.debug("[ENOTUM] Comprobando y validando la respuesta ...");
        if(respuestaSiri!=null && SiriUtils.validarRespuestaSiri(respuestaSiri)){
            LOG.debug("[ENOTUM] Respuesta desde Siri recibida correctamente ...");
            LOG.debug("[ENOTUM] Recuperando respuesta de Enotum ...");

            NTResposta respuestaEnotum = NTUtils.recuperarRespuestaEnotum(respuestaSiri);

            if(respuestaEnotum!=null && NTUtils.validarRespuestaEnotum(respuestaEnotum)){
                LOG.debug("[ENOTUM] Respuesta de Enotum recibida correctamente, extraemos el estado ...");

                DadesNotificacio datosNotificacion = respuestaEnotum.getNotificacio().getDadesNotificacio();

                if(datosNotificacion!=null){
                    String state = datosNotificacion.getIdEstat().getValue();

                    LOG.debug("[ENOTUM] Estado de la notificacion: "+state);

                    //Estados para la agenda trew@
                    if(stateAvailable.contains(state))
                        notificationState.setState(State.AVAILABLE);
                    else if(stateRead.contains(state))
                        notificationState.setState(State.READ);
                    else if(stateRejected.contains(state))
                        notificationState.setState(State.REJECTED);
                    else if(stateNotSubscribed.contains(state))
                        notificationState.setState(State.NOT_SUBSCRIBED);

                    //Se añade el nombre del estado al mensaje devuelto
                    notificationState.setMessage(datosNotificacion.getEstat());

                    //La fecha debera actualizarse cuando recibamos los valores posibles para los estados
                    notificationState.setDate(Calendar.getInstance());
                }
                else
                    LOG.error("[ENOTUM] Error al recuperar la informacion de Enotum de la respuesta ...");
            }
            else
                LOG.error("[ENOTUM] Error al recuperar la respuesta de Enotum ...");
        }
        else
            LOG.debug("[ENOTUM] Error al recuperar o validar la respuesta de Siri ...");

        LOG.debug("[ENOTUM] Saliendo del metodo de recuperacion de estado de Enotum");

        return notificationState;
    }

    public String getServiceCode() {
        // TODO Auto-generated method stub
        return null;
    }

    public boolean isUserActive(User user) throws NotificatorException {
        // TODO Auto-generated method stub
        return true;
    }

    public void setServiceCode(String serviceCode) {
        // TODO Auto-generated method stub

    }

    public void subscribe(User user) throws NotificatorException {
        // TODO Auto-generated method stub

    }
}