/**
 * Copyright 2012, Ayuntamiento de Tarragona
 *  
 * Desarrollador por  Guadaltel, S.A.
 * El presente cdigo fue desarrollado con financiacin del Ministerio 
 * de Industria Turismo y Comercio con el nmero de proyecto TSI-050100-2009-219.
 * 
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the 
 * European Commission  subseqent versions of the EUPL (the "Licence");  you 
 * may not use this work except in compliance with the Licence.
 *
 * You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software distributed 
 * under the  Licence is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
 * CONDITIONS OF ANY KIND, either express or implied. See the Licence for the 
 * specific language governing  permissions and limitations under the Licence.
 */
package es.guadaltel.framework.notificator.util;

import java.io.ByteArrayInputStream;
import java.math.BigInteger;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import net.aocat.nt.v2_1.ntpeticio.DadesConsulta;
import net.aocat.nt.v2_1.ntpeticio.DadesUsuari;
import net.aocat.nt.v2_1.ntpeticio.Destinataris;
import net.aocat.nt.v2_1.ntpeticio.Documents;
import net.aocat.nt.v2_1.ntpeticio.NTPeticio;
import net.aocat.nt.v2_1.ntpeticio.Notificacio;
import net.aocat.nt.v2_1.ntpeticio.Tramesa;
import net.aocat.nt.v2_1.ntpeticio.Tramesa.DadesOfici;
import net.aocat.nt.v2_1.ntresposta.Errors;
import net.aocat.nt.v2_1.ntresposta.NTResposta;
import net.gencat.scsp.esquemes.respuesta.Respuesta;

import org.apache.axis.encoding.Base64;
import org.apache.axis.utils.XMLUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import es.guadaltel.framework.core.User;
import es.guadaltel.framework.notificator.NotificatorTransaction;
import es.guadaltel.framework.register.Register;
import es.guadaltel.framework.register.RegisterFactory;
import es.guadaltel.framework.register.RegisterTransaction;
import es.guadaltel.framework.register.exception.RegisterException;


public class NTUtils {
    private static final Log LOG = LogFactory.getLog(NTUtils.class);

    public static Document crearDocumentoProcesarTramesa(NotificatorTransaction notificatorTransaction){
        LOG.debug("[ENOTUM] Entramos en el metodo crearDocumentoProcesarTramesa");

        Document docaux = null;

        try{
            LOG.debug("[ENOTUM] Rellenamos los datos de la solicitud a Enotum ...");

            //Datos especificos
            NTPeticio ntpeticio = new NTPeticio();
            ntpeticio.setCodiOperacio(SiriClient.getNotificatorProperty(ConstantsNotificator.COD_OPERACION_CREACION));

            //Dades usuari
            DadesUsuari dadesUsuario = new DadesUsuari();
            dadesUsuario.setNomUsuari(SiriClient.getNotificatorProperty(ConstantsNotificator.NOMBRE_USUARIO_DEFECTO));
            dadesUsuario.setRolUsuari(SiriClient.getNotificatorProperty(ConstantsNotificator.ROL_USUARIO_DEFECTO));
            dadesUsuario.setCodiOrganisme(SiriClient.getNotificatorProperty(ConstantsNotificator.COD_ORGANISMO_DEFECTO));
            dadesUsuario.setCodiDepartament(SiriClient.getNotificatorProperty(ConstantsNotificator.COD_DEPARTAMENTO_DEFECTO));

            ntpeticio.setDadesUsuari(dadesUsuario);

            //Tramesa
            Tramesa tramesa = new Tramesa();

                //Dades Ofici
                if(notificatorTransaction.getSubject().equals("") || notificatorTransaction.getText().equals(""))
                    throw new Exception("ERROR: Tanto el asunto de la notificación como el texto de la misma deben estar rellenos");

                DadesOfici dadesOfici = new DadesOfici();
                dadesOfici.setTitol(notificatorTransaction.getSubject());
                dadesOfici.setMissatge(notificatorTransaction.getText());
                dadesOfici.setPeuRecurs("No");

            tramesa.setDadesOfici(dadesOfici);

                //Documentos sin codificar en base64
                if(notificatorTransaction.getDocument()!=null){
                    LOG.debug("[ENOTUM] Incluimos documento en la solicitud ...");

                    Documents documents = new Documents();
                    Documents.Document document = new Documents.Document();

                    document.setNom(notificatorTransaction.getDocument().getName());
                    document.setTipus("Resolució");
                    document.setDades(Base64.encode(notificatorTransaction.getDocument().getContent()));

                    documents.getDocument().add(document);

                    tramesa.setDocuments(documents);
                }

                //Notificacion
                Notificacio notificacio = new Notificacio();


                //Comprobamos si se configuro el registro automátido en sigem de la salida
                if(SiriClient.getNotificatorProperty(ConstantsNotificator.REGISTRAR_SIGEM).equals("S")){
                    LOG.debug("[ENOTUM] La notificacion requiere alta en Sigem, se procede ...");

                    Register register = RegisterFactory.createInstance("sigem");

                    RegisterTransaction registerInfo = new RegisterTransaction();
                    registerInfo.setType(RegisterTransaction.TYPE_REGISTER_OUT);

                    try {
                        register.register(registerInfo);

                        notificacio.setReferencia("Ref. Sigem: "+registerInfo.getRecordNumber());
                        LOG.debug("[ENOTUM] Referencia del registro de salida en Sigem para la notificacion: "+registerInfo.getRecordNumber());
                    } catch (RegisterException e) {
                        LOG.error("[ENOTUM] Se produjo un error al realizar el registro de salida en Sigem");
                        e.printStackTrace();
                        return null;
                    }
                }
                else
                    notificacio.setReferencia(notificatorTransaction.getFileNumber());


                    //Destinataris
                    Destinataris destinataris = new Destinataris();
                    Destinataris.Destinatari destinatari = new Destinataris.Destinatari();

                    User user = notificatorTransaction.getUser();

                    LOG.debug("[ENOTUM] Validando los datos del usuario de destino ...");

                    if(!validarDatosUsuarioDestinatario(user))
                        throw new Exception("[ENOTUM] Error: El usuario destinatario no reune los datos necesarios para enviar la notificación");

                    destinatari.setNIF(user.getIdNumber());
                    destinatari.setNom(user.getName());
                    destinatari.setPrimerCognom(user.getSurname1());
                    destinatari.setSegonCognom(user.getSurname2());
                    destinatari.setBustiaCorreu(user.getEmail());

                    if(campoRelleno(user.getTelephone()))
                        destinatari.setTelefon(user.getTelephone());

                    destinataris.getDestinatari().add(destinatari);

                notificacio.setDestinataris(destinataris);

            tramesa.getNotificacio().add(notificacio);

            ntpeticio.setTramesa(tramesa);

            //Elaboramos el documento
            JAXBContext jc = JAXBContext.newInstance("net.aocat.nt.v2_1.ntpeticio");
            Marshaller marshaller = jc.createMarshaller();

            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();

            docaux = builder.newDocument();

            Element rootaux = docaux.createElement("ObjectNTP");

            LOG.debug("[ENOTUM] Creando marshal del xml de la solicitud a Enotum ...");

            marshaller.marshal(ntpeticio, rootaux);

            docaux.appendChild(rootaux);
        }
        catch(Exception ex){
            ex.printStackTrace();
        }

        LOG.debug("[ENOTUM] Salimos del metodo crearDocumentoProcesarTramesa");

        return docaux;
    }

    public static Document crearDocumentoConsultaNotificacion(long cod){
        LOG.debug("[ENOTUM] Entramos en el metodo crearDocumentoConsultaNotificacion");

        Document docaux = null;

        try{
            LOG.debug("[ENOTUM] Rellenamos los datos de la solicitud a Enotum ...");

            //Datos especificos
            NTPeticio ntpeticio = new NTPeticio();
            ntpeticio.setCodiOperacio(SiriClient.getNotificatorProperty(ConstantsNotificator.COD_OPERACION_CONSULTA));

            //Dades usuari
            DadesUsuari dadesUsuario = new DadesUsuari();
            dadesUsuario.setNomUsuari(SiriClient.getNotificatorProperty(ConstantsNotificator.NOMBRE_USUARIO_DEFECTO));
            dadesUsuario.setRolUsuari(SiriClient.getNotificatorProperty(ConstantsNotificator.ROL_USUARIO_DEFECTO));
            dadesUsuario.setCodiOrganisme(SiriClient.getNotificatorProperty(ConstantsNotificator.COD_ORGANISMO_DEFECTO));
            dadesUsuario.setCodiDepartament(SiriClient.getNotificatorProperty(ConstantsNotificator.COD_DEPARTAMENTO_DEFECTO));

            ntpeticio.setDadesUsuari(dadesUsuario);

            //Dades consulta
            DadesConsulta dadesConsulta = new DadesConsulta();
            dadesConsulta.setCodiNotificacio(BigInteger.valueOf(cod));

            ntpeticio.setDadesConsulta(dadesConsulta);

            //Elaboramos el documento
            JAXBContext jc = JAXBContext.newInstance("net.aocat.nt.v2_1.ntpeticio");
            Marshaller marshaller = jc.createMarshaller();

            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();

            docaux = builder.newDocument();

            Element rootaux = docaux.createElement("ObjectNTP");

            LOG.debug("[ENOTUM] Creando marshal del xml de la solicitud a Enotum ...");

            marshaller.marshal(ntpeticio, rootaux);

            docaux.appendChild(rootaux);
        }
        catch(Exception ex){
            ex.printStackTrace();
        }

        LOG.debug("[ENOTUM] Salimos del metodo crearDocumentoConsultaNotificacion");

        return docaux;
    }

    private static boolean validarDatosUsuarioDestinatario(User user){
        LOG.debug("[ENOTUM] Entrando en el metodo validarDatosUsuarioDestinatario");

        if(!campoRelleno(user.getIdNumber())){
            LOG.error("[ENOTUM] El campo idNumber del usuario esta vacio");
            return false;
        }

        if(!campoRelleno(user.getName())){
            LOG.error("[ENOTUM] El campo nombre del usuario esta vacio");
            return false;
        }

        if(!campoRelleno(user.getSurname1())){
            LOG.error("[ENOTUM] El campo apellido1 del usuario esta vacio");
            return false;
        }

        if(!campoRelleno(user.getSurname2())){
            LOG.error("[ENOTUM] El campo apellido2 del usuario esta vacio");
            return false;
        }

        if(!campoRelleno(user.getEmail())){
            LOG.error("[ENOTUM] El campo email del usuario esta vacio");
            return false;
        }

        return true;
    }

    private static boolean campoRelleno(String cad){
        return (cad!=null && !cad.equals(""));
    }

    public static NTResposta recuperarRespuestaEnotum(Respuesta respuestaSiri){
        LOG.debug("[ENOTUM] Entrando al metodo recuperarRespuestaEnotum");

        NTResposta ntResposta = null;

        try{
            Element ntElement = (Element) respuestaSiri.getTransmisiones().getTransmisionDatos().get(0).getDatosEspecificos();

            //Usamos ntElement.getFirstChild() para empezar a leer desde NTResposta, y no desde DatosEspecificos
            //Cambiamos el NS de NTv1 por el de NTv2.1 para que jaxb permita el unmarshall
            String content = XMLUtils.ElementToString((Element) ntElement.getFirstChild());
            content = content.replace("http://www.aocat.net/NT/NTResposta", "http://www.aocat.net/NT/v2.1/NTResposta");

            //Parsemos la respuesta especifica de enotum al modelo de datos
            JAXBContext jc = JAXBContext.newInstance("net.aocat.nt.v2_1.ntresposta");
            Unmarshaller unmarshaller = jc.createUnmarshaller();

            ntResposta = (NTResposta) unmarshaller.unmarshal(new ByteArrayInputStream(content.getBytes("UTF-8")));
        }
        catch(Exception e){
            LOG.error("[ENOTUM] Error: Se produjo un error al recuperar la respuesta de enotum: "+e.getMessage());
            e.printStackTrace();
        }

        LOG.debug("[ENOTUM] Respuesta Enotum recuperada");

        LOG.debug("[ENOTUM] Saliendo del metodo recuperarRespuestaEnotum");

        return ntResposta;
    }

    public static boolean validarRespuestaEnotum(NTResposta respuesta){
        LOG.debug("[ENOTUM] Entrando al metodo validarRespuestaEnotum");

        if(respuesta.getErrors()!=null && respuesta.getErrors().getError().size()>0){
            for(Errors.Error error : respuesta.getErrors().getError())
                LOG.error("[ENOTUM] Error [cod "+error.getCodiError()+"]: "+error.getMissatgeError());

            return false;
        }

        LOG.debug("[ENOTUM] Saliendo del metodo validarRespuestaEnotum");

        return true;
    }

    public static String recuperarIdNotificacionEnotum(NTResposta respuesta){
        String ntId = null;

        if(respuesta.getNotificacionsCreades().getCodiNotificacio().size()>0)
            ntId = respuesta.getNotificacionsCreades().getCodiNotificacio().get(0).toString();

        LOG.debug("[ENOTUM] Recuperado identificador de la peticion: "+ntId);

        return ntId;
    }
}