/**
 * Copyright 2012, Ayuntamiento de Tarragona
 *  
 * Desarrollador por  Guadaltel, S.A.
 * El presente cdigo fue desarrollado con financiacin del Ministerio 
 * de Industria Turismo y Comercio con el nmero de proyecto TSI-050100-2009-219.
 * 
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the 
 * European Commission  subseqent versions of the EUPL (the "Licence");  you 
 * may not use this work except in compliance with the Licence.
 *
 * You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software distributed 
 * under the  Licence is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
 * CONDITIONS OF ANY KIND, either express or implied. See the Licence for the 
 * specific language governing  permissions and limitations under the Licence.
 */
package es.guadaltel.framework.notificator.util;

import java.io.ByteArrayInputStream;
import java.io.StringWriter;
import java.net.URL;
import java.security.Security;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import net.aocat.siri.proxy.client.SIRIProxyClient;
import net.gencat.scsp.esquemes.peticion.Peticion;
import net.gencat.scsp.esquemes.respuesta.Respuesta;

import org.apache.axis.utils.XMLUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class SiriClient {
    private static final Log LOG = LogFactory.getLog(NTUtils.class);

    public Respuesta enviarPeticionSiri(Peticion peticion){
        Respuesta respuesta = null;

        try{
            //Se recupera el contenido del objeto jaxb
            JAXBContext jc = JAXBContext.newInstance("net.gencat.scsp.esquemes.peticion");
            Marshaller marshaller = jc.createMarshaller();
            StringWriter sw = new StringWriter();

            marshaller.marshal(peticion, sw);

            String xmlfinal = sw.toString();
            System.out.println("[SIRI REQUEST...]\n"+xmlfinal+"\n[...SIRI REQUEST]");

            //Se crea el documento de la peticion
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document xml = builder.parse(new ByteArrayInputStream(xmlfinal.getBytes("UTF-8")));

            //Realizamos la peticion al ws
            URL url= new URL(getNotificatorProperty(ConstantsNotificator.ENDPOINT));
            cargarContextoPeticiones();

            SIRIProxyClient proxy = new SIRIProxyClient();
            Element res = proxy.invoca(xml.getDocumentElement(),url,true);

            System.out.println("[SIRI RESPONSE...]\n"+XMLUtils.ElementToString(res)+"\n[...SIRI RESPONSE]");

            //Parseamos la respuesta del servidor al modelo de datos
            jc = JAXBContext.newInstance("net.gencat.scsp.esquemes.respuesta");
            Unmarshaller unmarshaller = jc.createUnmarshaller();

            respuesta = (Respuesta) unmarshaller.unmarshal(res);
        }
        catch(Exception e){
            LOG.error("[SIRI] ERROR - Se produjo un error al enviar la peticion: "+e.getMessage());
            e.printStackTrace();
        }

        return respuesta;
    }

    private void cargarContextoPeticiones(){
        // Configuracion socket TSL
        System.setProperty("javax.net.ssl.trustStore", getNotificatorProperty(ConstantsNotificator.TRUSTSTORE));
        System.setProperty("javax.net.ssl.trustStoreType", getNotificatorProperty(ConstantsNotificator.TRUSTSTORE_TYPE));
        System.setProperty("javax.net.ssl.trustStorePassord", getNotificatorProperty(ConstantsNotificator.TRUSTSTORE_PASSWORD));

        // Security Provider
        System.setProperty("java.protocol.handler.pkgs", "com.sun.net.ssl.internal.www.protocol");
        Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());
    }

    public static String getNotificatorProperty(String clave){
        try
        {
            ResourceBundle propiedades = ResourceBundle.getBundle("notificator");
            return propiedades.getString(ConstantsNotificator.PREFIX_COMPONENTE+clave);
        }
        catch(MissingResourceException mre)
        {
            LOG.error("[SIRI] ERROR: no se pudo recuperar el parametro de propiedades:"+ clave);
            return null;
        }
    }
}