/**
 * Copyright 2012, Ayuntamiento de Tarragona
 *  
 * Desarrollador por  Guadaltel, S.A.
 * El presente cdigo fue desarrollado con financiacin del Ministerio 
 * de Industria Turismo y Comercio con el nmero de proyecto TSI-050100-2009-219.
 * 
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by the 
 * European Commission  subseqent versions of the EUPL (the "Licence");  you 
 * may not use this work except in compliance with the Licence.
 *
 * You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software distributed 
 * under the  Licence is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
 * CONDITIONS OF ANY KIND, either express or implied. See the Licence for the 
 * specific language governing  permissions and limitations under the Licence.
 */
package es.guadaltel.framework.notificator.util;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import net.gencat.scsp.esquemes.peticion.Atributos;
import net.gencat.scsp.esquemes.peticion.DatosAutorizacion;
import net.gencat.scsp.esquemes.peticion.DatosGenericos;
import net.gencat.scsp.esquemes.peticion.Funcionario;
import net.gencat.scsp.esquemes.peticion.Peticion;
import net.gencat.scsp.esquemes.peticion.Solicitante;
import net.gencat.scsp.esquemes.peticion.SolicitudTransmision;
import net.gencat.scsp.esquemes.peticion.Solicitudes;
import net.gencat.scsp.esquemes.peticion.Transmision;
import net.gencat.scsp.esquemes.respuesta.Respuesta;


public class SiriUtils {
    private static final Log LOG = LogFactory.getLog(SiriUtils.class);

    public static Peticion crearPeticionSiriPorDefecto(){
        //Peticion
        Peticion peticion = new Peticion();

        peticion.setAtributos(crearAtributosPeticion());

            //Solicitudes
            Solicitudes solicitudes = new Solicitudes();

                //Solicitud Transmision
                SolicitudTransmision solicitud = new SolicitudTransmision();

                //Datos genericos
                solicitud.setDatosGenericos(SiriUtils.crearDatosGenericosSolicitud());

            solicitudes.getSolicitudTransmision().add(solicitud);

        peticion.setSolicitudes(solicitudes);

        return peticion;
    }

    public static boolean validarRespuestaSiri(Respuesta respuesta){
        if(!respuesta.getAtributos().getEstado().getLiteralError().equals("OK")){
            LOG.error("[ENOTUM] Error al validar respuesta siri: "+respuesta.getAtributos().getEstado().getLiteralError());

            return false;
        }

        return true;
    }

    private static Atributos crearAtributosPeticion(){
        //Atributos para la validacion de la peticion en Siri
        SimpleDateFormat formatoDeFechaAtributos = new SimpleDateFormat(SiriClient.getNotificatorProperty(ConstantsNotificator.FORMATO_FECHA_PETICION));

        Atributos atributos = new Atributos();
        atributos.setTimeStamp(formatoDeFechaAtributos.format(new Date()));
        atributos.setIdPeticion("TEST-ENOTUM-G3-"+Long.toString(System.currentTimeMillis()));
        atributos.setNumElementos(1);
        atributos.setCodigoCertificado(SiriClient.getNotificatorProperty(ConstantsNotificator.COD_CERTIFICADO_DEFECTO));
        atributos.setCodigoProducto(SiriClient.getNotificatorProperty(ConstantsNotificator.COD_PRODUCTO_DEFECTO));

            //Datos autorizacion
            DatosAutorizacion datosAutorizacion = new DatosAutorizacion();
            datosAutorizacion.setIdentificadorSolicitante(SiriClient.getNotificatorProperty(ConstantsNotificator.ID_SOLICITANTE_DEFECTO));
            datosAutorizacion.setNombreSolicitante(SiriClient.getNotificatorProperty(ConstantsNotificator.NOMBRE_SOLICITANTE_DEFECTO));
            datosAutorizacion.setFinalidad(SiriClient.getNotificatorProperty(ConstantsNotificator.FINALIDAD_DEFECTO));

        atributos.setDatosAutorizacion(datosAutorizacion);

        return atributos;
    }

    private static DatosGenericos crearDatosGenericosSolicitud(){
        DatosGenericos datosGenericos = new DatosGenericos();

            //Solicitante
            Solicitante solicitante = new Solicitante();
            solicitante.setIdentificadorSolicitante(SiriClient.getNotificatorProperty(ConstantsNotificator.ID_SOLICITANTE_DEFECTO));
            solicitante.setNombreSolicitante(SiriClient.getNotificatorProperty(ConstantsNotificator.NOMBRE_SOLICITANTE_DEFECTO));
            solicitante.setFinalidad(SiriClient.getNotificatorProperty(ConstantsNotificator.FINALIDAD_DEFECTO));
            solicitante.setConsentimiento("Si");

                //Funcionario
                Funcionario funcionarioSolicitante = new Funcionario();
                funcionarioSolicitante.setNifFuncionario(SiriClient.getNotificatorProperty(ConstantsNotificator.NIF_FUNCIONARIO_DEFECTO));
                funcionarioSolicitante.setNombreCompletoFuncionario(SiriClient.getNotificatorProperty(ConstantsNotificator.NOMBRE_FUNCIONARIO_DEFECTO));

            solicitante.setFuncionario(funcionarioSolicitante);

        datosGenericos.setSolicitante(solicitante);

            //Transmision
            Transmision transmision = new Transmision();
            transmision.setCodigoCertificado(SiriClient.getNotificatorProperty(ConstantsNotificator.COD_CERTIFICADO_DEFECTO));
            transmision.setIdSolicitud("1");
            transmision.setIdTransmision("EXPEDIENT");

            SimpleDateFormat formatoDeFechaTransmision = new SimpleDateFormat("ddMMyyyy");
            transmision.setFechaGeneracion(formatoDeFechaTransmision.format(new Date()));

        datosGenericos.setTransmision(transmision);

        return datosGenericos;
    }
}